/* Minimal working example for the Frolov cubature points on $[0,1]^d$*/

#include <vector>
#include <functional>
#include <algorithm>
#include <exception>

#include <string>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>

/////////////////////
//customizable part//
/////////////////////

// location of the Frolov files
std::string directory = "FrolovFiles/";

/* Test integrand has homogeneous boundary conditions and Besov smoothness $r=2$ */
double testIntegrand(size_t d, std::vector<double> x){
  if(d!=x.size()) throw std::logic_error(
        "In testIntegrand: given dimension d and dimension of x do not match\n"
    );

  /* You can change the integrand if you want.
   * However, the Frolov method will recognize the Besov smoothness of the integrand
   * after it is extended by zero to the whole space $\mathbb{R}^d$.
   * If the integrand does not have homogeneous boundary conditions,
   * its Besov smoothness will most likely be r<=1. */

  double product = 1;
  for(int i=0; i<d; i++) product *= (1-x[i])*x[i];
  return product*std::pow(6.,d);
};
/* If you change the integrand,
 * you have to change the exact result as well.
 * Or obtain a very precise approximation in some other way. */
double exactResult(size_t d) {
  return 1.;
};
/* short description of the test integrand */
std::string description = "$ 6^d \\prod_{i=1}^d (1-x_i)x_i $";

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main() {
  size_t d; //dimension
  std::cout << "This is the minimal working example for Frolov cubature on the unit cube [0,1]^d. \n"
            << "Please specify the dimension d (from 2 to 10): ";
  std::cin >> d;
  std::cout << std::endl << std::endl;
  if(d<2 || d>10) throw std::logic_error("invalid choice of dimension d...\n");

  double w ;//uniform weight
  int N; //number of integration nodes
  std::vector<double> x(d);

  std::vector<int> integrationNodes(20);
  std::vector<double> cubatureResults(20,0.0);
  std::vector<double> integrationErrors(20,0.0);

  std::cout
      << "Number of integration nodes, cubature result and integration error "
      << "for the test integrand "
      << description
      << "(as specified in MWE.cpp):\n";

  bool missingFiles=false;
  /* the actual cubature: for scaling factors n=1,2,4,...,2^20,
   * load integration nodes and weight, evaluate the function,
   * compute weighted sum*/
  for(int n=1; n < 21; n++) {
    std::ifstream read;
    std::string filename = directory + "Frolov_"
        + std::to_string(d)
        + '_' + std::to_string(n)
        + "_basic.dat";
    std::string line;
    read.open(filename);
    if(!read.is_open()) {
      std::cout << "file '" << filename << "' could not be opened...\n";
      missingFiles = true;
      continue;
    }
    std::getline(read,line);std::getline(read,line); std::getline(read,line);
    N = std::stoi(line);
    integrationNodes[n-1] = N;
    std::getline(read,line);
    w = std::stod(line);
    std::vector<double> functionEvaluations(N);
    for(size_t i=0; i<N; i++) {
      std::getline(read,line);
      std::istringstream iss(line);
      for(int k=0; k<d;k++) {
        iss >> x[k];
      }
      functionEvaluations[i] = testIntegrand(d,x);
    }
    std::sort(functionEvaluations.begin(),functionEvaluations.end());
    for(size_t i=0; i<N; i++) cubatureResults[n-1] += functionEvaluations[i];
    cubatureResults[n-1] *= w;
    integrationErrors[n-1] = std::abs(cubatureResults[n-1]-exactResult(d));
    std::cout << N << ' '
              << std::setprecision(16)
              << cubatureResults[n-1] << ' ' << integrationErrors[n-1] << std::endl;
    read.close();
  }
  std::cout << std::endl;
  std::cout << "(Some integration files were missing or corrupted. "
            << "You might want to download them from the homepage:\n"
            << "http://wissrech.ins.uni-bonn.de/research/software/frolov/)\n";

  return 0;
};